// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancysslpkey.h 854 2011-01-05 21:11:05Z fancylibs $
**
** Copyright (C) 2003 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** A class for handling OpenSSL private keys.
**
*****************************************************************************/

#ifndef FANCYSSL_PKEY_H
#define FANCYSSL_PKEY_H

#include <fancysslbase.h>

// fwd decls from openssl.
extern "C" {
  struct evp_pkey_st;
  typedef struct evp_pkey_st EVP_PKEY;
};

namespace Fancy {

template<class T> class ArrayObject;

  namespace SSL {

    /**
     * A representation of an OpenSSL private key.
     */
    class FANCYSSL_API PKey : public Base
    {
    protected:
    	PKey (EVP_PKEY *k);

    	~PKey();
    public:

    	typedef Fancy::ObjPtr<PKey> ptr_t;

        /**
          Aquires the given OpenSSL handle and automagically frees the handle,
          if the object goes out of scope. The caller must not call \c PKEY_free
          on the supplied resource.
        */
    	static ptr_t newInstance(EVP_PKEY *k);

    	/**
    	 * Read a private key from a PEM-encoded string.
    	 * @param passwd The password used to decode the string or @c NULL, if the
    	 *               the string is expected to be unencrypted.
    	 * @param pem The PEM string.
    	 * @param len The length of @c pem or -1,  if the string is null-terminated.
    	 * @return A new @c PKey instance.
    	 */
    	static ptr_t newInstance(const char *passwd, const char *pem, int len=-1);

    	/**
    	 * Read a private key from a ASN1 block.
    	 * @param asn1 The ASN1 block.
    	 * @param len  The length of @c asn1.
    	 * @return A new @c PKey instance.
    	 */
    	static ptr_t newRSAInstance(const unsigned char *asn1, int len);

    	/**
    	 * Read a private key from a PEM-encoded file.
    	 * @param passwd dThe password used to decode the file or @c NULL, if the
    	 *               the string is expected to be unencrypted.
    	 * @param file The path to the PEM-encoded file.
    	 * @return A new @c PKey instance.
    	 */
    	static ptr_t readFile(const char *file, const char *passwd);

    	/**
    	 * Generate an RSA private key using a given number of bits.
    	 * @param nbits the number of bits for the generated key.
    	 * @return A new @PKey instance.
    	 */
    	static ptr_t generateRSA(int nbits);

    	EVP_PKEY *getPKEY () const;

    	/**
    	 * PEM-enccode this key.
    	 * @param passwd A password to encode the key or @c NULL, if the key
    	 *               should not be encoded.
    	 * @return A PEM-encoded representation of this key.
    	 */
    	std::string getPEMString(const char *passwd) const;


    private:
    	PKey (const PKey &a);
    	PKey &operator= (const PKey &a);

        EVP_PKEY *pkey;
    };
  }
}

#endif
