// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancysslsockdev.h 819 2010-09-09 12:02:31Z fancylibs $
**
** Copyright (C) 2003 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** A base class, which wraps TCP/IP sockets for use with libfancy.
** Extensible for using SSL or socks etc. by using many virtual functions.
**
*****************************************************************************/

#ifndef FANCYSSL_SOCK_DEVICE_H
#define FANCYSSL_SOCK_DEVICE_H

#include <fancysockdevice.h>
#include <fancysslbase.h>

extern "C"
{
  struct ssl_ctx_st;
  typedef struct ssl_ctx_st SSL_CTX;

  struct ssl_st;
  typedef struct ssl_st SSL;
};

namespace Fancy {

class String;

  namespace SSL {

    class X509Cert;
    class X509Crl;
    class PKey;

    class FANCYSSL_API SockCtx : public Base, virtual public Fancy::Sock::DeviceFactory
    {
    public:
    	typedef Fancy::ObjPtr<SockCtx> ptr_t;

    	enum Method     { SSLv2, SSLv3, SSLv23, TLSv1 };
    	enum Type       { Client, Server };
    	enum PeerPolicy { NoPeerCert, RequirePeerCert, VerifyPeerCert };

    	static ptr_t newInstance(Method m, Type t, PeerPolicy p);

    	void setSessionID(const std::string &id);

    	PeerPolicy getPeerPolicy() const;
    	Type getType() const;

    	Fancy::ObjPtr<X509Cert> getCertificate() const;

    	void addVerifyCA(const Fancy::ObjPtr<X509Cert> &ca);
    	void addClientCA(const Fancy::ObjPtr<X509Cert> &ca);

    	void addVerifyCrl(const Fancy::ObjPtr<X509Crl> &crl);

    	void setCertificate(const Fancy::ObjPtr<X509Cert> &x509, const Fancy::ObjPtr<PKey> &pkey);

    	SSL_CTX *getSSL_CTX() const;

    	// override DeviceFactory

    	/**
    	 * If no setSoRecvTimeout(int > 0) has been issued before, this may run into blocking reads!
    	 */
    	Fancy::Sock::Device::ptr_t connect(const Fancy::Sock::Address::ptr_t &addr);

    	/**
    	 * If no setSoRecvTimeout(int > 0) has been issued before, the accept() of the created SSL::Sock::Device may run into blocking reads!
    	 * The current implementation still cannot serve new clients while waiting for a bad client to time out.
    	 */
    	Fancy::Sock::Device::ptr_t bind(const Fancy::Sock::Address::ptr_t &addr);

    	/**
    	 * If ms>0, the recv timeout is applied to all new sockets right after raw connect.
    	 */
    	void setSoRecvTimeout(int ms);

    	/**
    	 * Returns the value previously set by setSoRecvTimeout(int).
    	 */
    	int getSoRecvTimeout();

    protected:
        SockCtx(Type t, PeerPolicy _pp, SSL_CTX *_ctx);
        ~SockCtx();

    private:
    	Type type;
    	PeerPolicy pp;
    	Fancy::ObjPtr<X509Cert> my_cert;
    	SSL_CTX *ctx;
    	int factorySoRecvTimeout;
    };

    class FANCYSSL_API SockDevice : public Fancy::Sock::Device
    {
    public:
      SockDevice(const SockCtx::ptr_t &ctx, const Fancy::Sock::Device::ptr_t &dev, const Fancy::ObjPtr<X509Cert> &peer, ::SSL *_ssl);

      ~SockDevice();

    public:
    	typedef Fancy::ObjPtr<SockDevice> ptr_t;

    	/**
    	 * Wrap a given socket device.
    	 *
    	 * @param ctx The SSL context to use.
    	 * @param dev The device to be used as SSL transport.
    	 * @return An SSL socket device on top of the given device.
    	 */
    	static ptr_t newInstance (const SockCtx::ptr_t &ctx, const Fancy::Sock::Device::ptr_t &dev);

    	/**
    	 * Connect to the given given address using a given SSL context.
    	 *
    	 * @param ctx The SSL context to use.
    	 * @param addr The socket address to connect to.
    	 * @return An SSL socket device connected to the given address.
    	 */
    	static ptr_t connect (const SockCtx::ptr_t &ctx, const Fancy::Sock::Address::ptr_t &addr);

    	/**
    	 * Bind to the given given address using a given SSL context.
    	 *
    	 * @param ctx The SSL context to use.
    	 * @param addr The socket address to bind to.
    	 * @return An SSL socket device bound to the given address.
    	 */
    	static ptr_t bind (const SockCtx::ptr_t &ctx, const Fancy::Sock::Address::ptr_t &addr);

    	/**
    	 * @return The OpenSSL SSL pointer wrapped by this @c SockDevice instance.
    	 */
    	::SSL *getSSL() const;

    	/**
    	 * @return The SSL context used by this device.
    	 */
    	SockCtx::ptr_t getSockCtx() const;

    	/**
    	 * @return The certificate presented by the peer during the initial handshake or
    	 *         @c null, if we did not receive such a certificate.
    	 */
    	Fancy::ObjPtr<X509Cert> getPeerCert() const;

    	/**
    	 * @return The underlying physical device.
    	 */
    	Fancy::Sock::Device::ptr_t getDevice() const;

    	// reimplemented functions from
    	// Fancy::Channel, Fancy::PollableChannel, Fancy::Sock::Device
    	Fancy::Sock::Device::ptr_t accept();
    	void listen (int backlog);

    	void shutdown(Fancy::ShutdownOption o);
    	void close();

    	int read (void *buf, int count);
    	int write (const void *buf, int count);

    	void flush();

    	bool isOpen() const;

    	void setSoRecvTimeout(int ms);
    	int getSoRecvTimeout() const;

#ifdef WIN32
    	void *getEventHandle();
    	int prePoll(PollEvent *events);
    	int postPoll(PollEvent *events);
#else
    	int getPollFileDescriptor() const;
#endif

    	size_t bytesAvailable() const;

    	Fancy::Sock::Address::ptr_t getAddr() const;

    	std::string toString() const;

    	/**
    	 * @return the key used to attach the peer certificate to the
    	 *         authentication info metadata in @c addAuthenticationInfo
    	 */
    	static Fancy::ObjPtr<Fancy::String> getX509CertKey();

    	void putAuthenticationInfo(Fancy::ObjPtr<Fancy::Map> &metadata) const;

    private:
    	SockDevice(const SockDevice &);
    	SockDevice &operator= (const SockDevice &);

    	SockCtx::ptr_t sock_ctx;
    	Fancy::Sock::Device::ptr_t device;
    	Fancy::ObjPtr<X509Cert> peer_cert;
    	::SSL *ssl;
    };

  }
}

#endif // ! FANCYSSL_SOCK_DEVICE_H
